package examples.hibernate.domain;

import javax.persistence.*;

import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;


@Entity
@Inheritance(strategy = InheritanceType.JOINED)
public class Airplane extends Aircraft {    // persistent class, root of a persistent class hierarchy
    public enum AeroplaneState { MAINTENANCE_DUE, FLYING, GROUNDED }

    private static int aeroplaneCount;       // transient property
    private transient double weightInPounds; // transient property

    @Transient
    private String currentPilot;             // transient property
    
    private long wingSpan;                   // persistent property
    private long length;                     // persistent property
    
    @Enumerated(EnumType.STRING)
    private AeroplaneState state;            // enum persisted as String in database
    
    @Temporal(TemporalType.DATE)
    private Date lastMaintenance;            // persistent property
    
    @Lob
    private String specs;                    // persistent property persisted as a CLOB

    @ManyToMany
    private Set<Passenger> passengers;       // many-to-many bidirectional association (owner)
    
    protected Airplane() { }
    
    public Airplane(String name, long weigth, long wingSpan, long length) {
	super(name, weigth);
	aeroplaneCount++;
	setWingSpan(wingSpan);
	setLength(length);
	setState(AeroplaneState.FLYING);
	setLastMaintenance(new Date());
	passengers = new HashSet<Passenger>();
    }

    public static long getAeroplaneCount() {
	return aeroplaneCount;
    }

    @Override
    public void setWeight(long weight) {
	super.setWeight(weight);
	weightInPounds = weight * 2.20462262185;
    }

    public double getWeightInPounds() {
	return weightInPounds;
    }

    public String getCurrentPilot() {
	return currentPilot;
    }
    
    public void setCurrentPilot(String currentPilot) {
	this.currentPilot = currentPilot;
    }

    public Date getLastMaintenance() {
	return new Date(lastMaintenance.getTime());
    }
    
    public void setLastMaintenance(Date date) {
	this.lastMaintenance = date;
    }

    public void resetLastMaintenance() {
	this.lastMaintenance = new Date();
    }

    public AeroplaneState getState() {
	return state;
    }

    public void setState(AeroplaneState state) {
	this.state = state;
    }

    public long getLength() {
	return length;
    }

    public void setLength(long length) {
	this.length = length;
    }
    
    public long getWingSpan() {
	return wingSpan;
    }

    public void setWingSpan(long wingSpan) {
	this.wingSpan = wingSpan;
    }

    public String getSpecs() {
	return specs;
    }
    
    public void setSpecs(String specs) {
	this.specs = specs;
    }

    public Set<Passenger> getPassengers() {
	return Collections.unmodifiableSet(passengers);
    }
    
    public void addPassenger(Passenger passenger) {
	this.passengers.add(passenger);
	passenger.addAirplane(this);
    }

    public void removePassenger(Passenger passenger) {
	this.passengers.remove(passenger);
	passenger.removeAirplane(this);
    }

    @Override
    public String toString() {
	StringBuilder sb = new StringBuilder(super.toString()).append("\n\tLength:")
		.append(length).append("m")
		.append("\n\tWing span: ").append(wingSpan).append("m")
		.append("\n\tSpecifications: ")
		.append(specs)
		.append("\n\tMaintenance State: ")
		.append(state)
		.append("\n\tLast Maintenance: ")
		.append(SimpleDateFormat.getDateInstance().format(lastMaintenance));
	
	if (!passengers.isEmpty()) {
	    sb.append("\n\tPassengers:");
	    for (Passenger passenger : passengers) {
		sb.append("\n").append(passenger).append("\n\t\t---");
	    }
	}

	return sb.toString();
    }

}
